import javax.swing.*;
import java.awt.*;

/**
 * This class is a widget that animates a sort algorithm, by showing
 * the integers as bars of a size corresponding to the size of the integer.
 *
 * An unsorted vector is shown in red, a sorted vector in green,
 * Indeces that are being compared are shown in blue
 * 
 * 
 * @author Sara Stymne
 *
 */
public class AnimationWidget extends JComponent {
	
    /**
	 * Needed beacuse this class implements the Serializable interface
	 */
	private static final long serialVersionUID = 1L;
	
	/**
	 * A reference to the vector that is sorted, which is animated by this class
	 */
	private SortVector vector;
	
	/**
	 * The name of the algorithm
	 */
	private String algorithmName;
	
	private Color barColor;
	private Color compareColor;

	private int barSize;
	private int barHeight;
	
	
	/**
	 * Creates an instance of AnimationWidget with 
	 * a given vector and an algorithm name
	 * 
	 * it will probably have to do more things!
	 * 
	 * @param vector the sortVector to draw
	 * @param algorithmName the name of the algorithm
	 */
	public AnimationWidget(SortVector vector,
			               String algorithmName) {
		this.vector = vector;
		this.algorithmName = algorithmName;
        
        barColor = Color.RED;

		compareColor = Color.BLUE;
		
		//You will probably need to do some more stuff here
        setPreferredSize(new Dimension(550, 250));
        repaint();
	}
	
		

      public void setVector(SortVector v) {
        vector = v;
        barColor = Color.RED;
        repaint();
      }

      public void paintComponent(Graphics g) {
    	  
        g.setColor(Color.white);
        Dimension d = getSize();
        g.fillRect(0,0, d.width, d.height);
        g.setColor(barColor);
        g.drawString(algorithmName, 15, 15);
        
        int bars = vector.getSize();
        int max = vector.getMax();     
        barSize = (d.width -10)/bars - 2;
        barHeight = (d.height - 10)/max;
       
        for(int i = 0; i < vector.size(); i++) {

          g.fillRect(i*barSize+2*i+5, (d.height)-vector.get(i)*barHeight, 
        		  barSize, vector.get(i)*barHeight);
        }

      }


	public void showCompared(int index1, int index2) {
		Graphics g = getGraphics();
		Dimension d = getSize();
		g.setColor(compareColor);
		g.fillRect(index1*barSize+2*index1+5, (d.height)-vector.get(index1)*barHeight, 
				   barSize, vector.get(index1)*barHeight);
		g.fillRect(index2*barSize+2*index2+5, (d.height)-vector.get(index2)*barHeight, 
				   barSize, vector.get(index2)*barHeight);
	}

      
      public void done() {
    	  barColor = Color.GREEN;
    	  repaint();
      }

	  public void prepareForSorting() {
    	  barColor = Color.RED;
    	  repaint();
      }

}
