import javax.swing.*;
import java.awt.*;


/**
 * The main class for the sorting animation. 
 * Holds the main window for the animation and 
 * the different algorithms that are animated
 *
 * This class does not have to be changed.
 * 
 * @author Sara Stymne
 *
 */
public class Animator {
	
	/**
	 * The algorithms that are animated
	 */
	private Algorithms algorithms; 
	
	/**
	 * The main copy of the vector of integers that are to be sorted
	 */
	private SortVector sortVector;
	
	/**
	 * The main window
	 */
	private JFrame frame;
	
	/**
	 * The panel for the input part
	 */
	private ButtonPanel buttonArea;
		
	/**
	 * The panel showing the sorting animations
	 */
	private JPanel sortingArea;
	
    
	/**
	 * Creates a new instance of the Animator
	 * that will place the algorithms on top of each other
	 */
	public Animator() {
		this(0,1);
	}
	
	/**
	 * Creates a new instance of the Animator
	 * with the algorithms placed in a grid with size 
	 * XCoord * YCoord
	 * 
	 * @param XCoord the x coordinate of the grid of algorithm widgets
	 * @param YCoord the y coordinate of the grid of algorithm widgets
	 */
	public Animator(int XCoord, int YCoord) {
		sortVector = new ScrambleVector().createNewInstance(50, 50);
		algorithms = new Algorithms(); 
		
		frame = new JFrame("animated sorting");
		frame.setLayout(new BorderLayout());
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		buttonArea = new ButtonPanel(this); 
        buttonArea.setDefaultVectorDims(50, 50);
		sortingArea = new JPanel(new GridLayout(XCoord, YCoord));
		
		frame.add(sortingArea, BorderLayout.CENTER);
		frame.add(buttonArea, BorderLayout.SOUTH);
	}    
	
    
    public void setNewVector(SortVector newVector) {
        if (algorithms.animationFinished()) {
            algorithms.setVector(newVector);
            frame.repaint();
        }
    }
	

	
	
	/**
	 * Starts the animation if it is not already running
	 */
	public void runAnimation(int swapWeight)  {
		if (algorithms.animationFinished()) {  
			algorithms.start(swapWeight);
		}
	}
	
	/**
	 * Shows the main frame
	 */
	public void showFrame() {
		frame.pack();
		frame.setVisible(true);
	}
	
	/**
	 * Adds a new AlgorithmHandler for the given algorithm
	 * 
	 * @param alg the new algorithm
	 */
	public void addAlgorithm(SortAlgorithm alg) {
		
		//Creates a copy of the vector for the new algorithm
		SortVector vectorClone = (SortVector) sortVector.clone();
		
		//adds the algorithm and its vector copy to the algorithm vector
		algorithms.add(alg, vectorClone);
		
		//adds the animation widget of the new algorithm to the sorting area.
		sortingArea.add(algorithms.getLastWidget());
		
	}

    /**
     * Sets the delay time 
     * that decides how long the break between each step of the animation is
     * 
     * @param dTime the new delay time
     */
    public void setDelayTime(int dTime) {
        algorithms.setDelayTime(dTime);      
    }

			
}
