
/**
 * A crude implementation of quicksort that chooses the pivot as the median
 * of the first, last and middle element.
 * (Also contains code where the left elements is used as pivot)
 * 
 * See this class as a sample of how other algorithms should be implemented.
 * It does not have to be changed.
 * 
 * @author Sara Stymne
 *
 */
public class QuickSortAlgorithm extends SortAlgorithm {

    public QuickSortAlgorithm() {
        super("Quick sort");
    }

     /**
	 * Sorts the vector using quick sort and the median of three as pivot
	 * 
	 * The sort() function should always be the main sorting 
     * function of each algorithm
	 * 
	 * @see SortAlgorithm#sort()
	 */
    protected void sort() {
        quickSortMedian(0, elementCount()-1);
    }
    

    private void quickSort(int scanRight, int scanLeft) {
        int scanToRight;
        int scanToLeft;

        if (scanRight >= scanLeft) {
			return;
		}
        scanToRight = scanRight;
        scanToLeft = scanLeft - 1;
        while (scanToRight <= scanToLeft) {
			while ((scanToRight <= scanToLeft) && (cmp(scanToRight, scanLeft) < 0)) {
				scanToRight++;
			}
			while ((scanToLeft >= scanToRight) && (cmp(scanToLeft, scanLeft) > 0)) {
				scanToLeft--;
			}
			if (scanToRight < scanToLeft) {
				swap(scanToRight,scanToLeft);
			}
        }
        swap(scanToRight,scanLeft);
        quickSort(scanRight, scanToRight - 1);
        quickSort(scanToRight + 1, scanLeft);
	}

	
    private void quickSortMedian(int scanRight, int scanLeft) {
        int scanToRight;
        int scanToLeft;

		//System.out.println("1: " + scanRight + " " + scanLeft);

        if (scanRight >= scanLeft) {
			return;
		}

		if (scanRight == scanLeft+1) {
			if (cmp(scanRight, scanLeft) > 0) {
				swap(scanRight, scanLeft);
			}
			return;
		}

		
        scanToRight = scanRight;
        scanToLeft = scanLeft - 1;

		
		//System.out.println("2: " + (scanLeft+(scanRight-scanLeft)/2));
				
		int pivot = median3(scanRight, scanLeft, scanLeft+(scanRight-scanLeft)/2);

		swap(scanLeft, pivot);

		//System.out.println("3: " + pivot);
		
		
        while (scanToRight <= scanToLeft) {
			while ((scanToRight <= scanToLeft) && (cmp(scanToRight, scanLeft) < 0)) {
				scanToRight++;
			}
			while ((scanToLeft >= scanToRight) && (cmp(scanToLeft, scanLeft) >= 0)) {
				scanToLeft--;
			}
			if (scanToRight < scanToLeft) {
				swap(scanToRight,scanToLeft);
			}
        }
        swap(scanToRight,scanLeft);
        quickSortMedian(scanRight, scanToRight - 1);
        quickSortMedian(scanToRight + 1, scanLeft);
	}

	private int median3(int i, int j, int k) {
        return (cmp(i,j) < 0 ?
               (cmp(j,k) < 0 ? j : cmp(i,k) < 0 ? k : i) :
               (cmp(k,j) < 0 ? j : cmp(k,i) < 0 ? k : i));
    }

}
