
import java.util.*;

/**
 * Class for handling a vector of Integers, 
 * that is being sorted in the animation
 *
 * This class does not have to be changed.
 * 
 * @author Sara Stymne
 *
 */
public abstract class SortVector extends Vector<Integer> {
    
   
    
    /**
     * The size of the sortVector
     */
    protected int size;
    
    /**
     * The maximum size of the numbers in the vector, 
     * only used when the vector is randomized
     */
    protected int max;
    
    
    /**
     * Creates an empty sortVector,
     * with size and max set to 50 
     */
    public SortVector() {
      this(50, 50);
    }

    /**
     * Creates an empty vector with size and max specified
     *
     * @param size the size of the vector
     * @param max the max size of the ntegers in the vector (when randomized)
     */
   public SortVector(int size, int max) {
      super();
      this.max = max;
      this.size = size;
    }
   
   
    
    /**
     * Changes the size of the vector next time it is reorganized
     * 
     * @param s the new size
     */
    public void setSize(int s) {
        size = s;
    }
    
    /**
     * Changes the maximum size of the numbers in the vector,
     * next time the vector is randomized
     * 
     * @param m the new max size
     */
    public void setMax(int m) {
        max = m;
    }
    
    
    /**
     * Gives the current max of the vector
     * 
     * @return Returns the max.
     */
    public int getMax() {
        return max;
    }

    /**
     * gives the current size of the vector
     * 
     * @return the size of the vector
     */
    public int getSize() {
        return size;
    }
    
    /**
     * same as getSize(). Exists for consistency with SortAlgorithm
     * 
     * @return the size of the vector
     */
    public int elementCount() {
        return size;
    }

    
    
    /**
     * Swaps the values of the numbers at the two given indeces in 
     * the vector
     * 
     * @param i index 1
     * @param j index 2
     */
    public void swap(int i, int j) {
        Integer tmp = get(i);
        set(i, get(j));
        set(j, tmp);
    }
    
    
 
    
    /**
     * Compares the values of the numbers at the two given indeces 
     * in the vector
     * 
     * @param i the index of the first number to compare
     * @param j the index of the second number to compare
     * @return   a negative value if the value at i is smaller
     *           a positive value if the value at i is bigger
     *           zero if the numbers at i and j are equal
     */
    public int cmp(int i, int j) {
        return get(i).intValue()-get(j).intValue();
    }
    
    /**
     * creates a new instance of the vector with the configuration of the
     * class it belongs to
     * 
     * @param size the size of the new instance
     * @param max the maximum number in the new instance
     * @return the new instance
     */
    public abstract SortVector createNewInstance(int size, int max);
    
    /** 
     * Returns a copy of the vector
     * 
     * @return the copy
     */
    public abstract SortVector clone();
    
}
